@@ -43,11 +43,23 @@ This module uses the FIBMAP ioctl to detect holes.
 # pylint: disable=R0902,R0903
 
 import hashlib
+import logging
 from bmaptools.BmapHelpers import human_size
-from bmaptools import Fiemap
+from bmaptools import Filemap
 
-# The bmap format version we generate
-SUPPORTED_BMAP_VERSION = "1.3"
+# The bmap format version we generate.
+#
+# Changelog:
+# o 1.3 -> 2.0:
+#   Support SHA256 and SHA512 checksums, in 1.3 only SHA1 was supported.
+#   "BmapFileChecksum" is used instead of "BmapFileSHA1", and "chksum="
+#   attribute is used instead "sha1=". Introduced "ChecksumType" tag. This is
+#   an incompatible change.
+#   Note, bmap format 1.4 is identical to 2.0. Version 1.4 was a mistake,
+#   instead of incrementing the major version number, we incremented minor
+#   version number. Unfortunately, the mistake slipped into bmap-tools version
+#   3.0, and was only fixed in bmap-tools v3.1.
+SUPPORTED_BMAP_VERSION = "2.0"
 
 _BMAP_START_TEMPLATE = \
 """<?xml version="1.0" ?>
@@ -93,7 +105,7 @@ class Error(Exception):
     """
     pass
 
-class BmapCreate:
+class BmapCreate(object):
     """
     This class implements the bmap creation functionality. To generate a bmap
     for an image (which is supposedly a sparse file), you should first create
@@ -106,6 +118,81 @@ class BmapCreate:
     the FIEMAP ioctl to generate the bmap.
     """
 
+    def __init__(self, image, bmap, chksum_type="sha256", log=None):
+        """
+        Initialize a class instance:
+        * image  - full path or a file-like object of the image to create bmap
+                   for
+        * bmap   - full path or a file object to use for writing the resulting
+                   bmap to
+        * chksum - type of the check sum to use in the bmap file (all checksum
+                   types which python's "hashlib" module supports are allowed).
+        * log     - the logger object to use for printing messages.
+        """
+
+        self._log = log
+        if self._log is None:
+            self._log = logging.getLogger(__name__)
+
+        self.image_size = None
+        self.image_size_human = None
+        self.block_size = None
+        self.blocks_cnt = None
+        self.mapped_cnt = None
+        self.mapped_size = None
+        self.mapped_size_human = None
+        self.mapped_percent = None
+
+        self._mapped_count_pos1 = None
+        self._mapped_count_pos2 = None
+        self._chksum_pos = None
+
+        self._f_image_needs_close = False
+        self._f_bmap_needs_close = False
+
+        self._cs_type = chksum_type.lower()
+        try:
+            self._cs_len = len(hashlib.new(self._cs_type).hexdigest())
+        except ValueError as err:
+            raise Error("cannot initialize hash function \"%s\": %s" %
+                        (self._cs_type, err))
+
+        if hasattr(image, "read"):
+            self._f_image = image
+            self._image_path = image.name
+        else:
+            self._image_path = image
+            self._open_image_file()
+
+        if hasattr(bmap, "read"):
+            self._f_bmap = bmap
+            self._bmap_path = bmap.name
+        else:
+            self._bmap_path = bmap
+            self._open_bmap_file()
+
+        try:
+            self.filemap = Filemap.filemap(self._f_image, self._log)
+        except (Filemap.Error, Filemap.ErrorNotSupp) as err:
+            raise Error("cannot generate bmap for file '%s': %s"
+                        % (self._image_path, err))
+
+        self.image_size = self.filemap.image_size
+        self.image_size_human = human_size(self.image_size)
+        if self.image_size == 0:
+            raise Error("cannot generate bmap for zero-sized image file '%s'"
+                        % self._image_path)
+
+        self.block_size = self.filemap.block_size
+        self.blocks_cnt = self.filemap.blocks_cnt
+
+    def __del__(self):
+        """The class destructor which closes the opened files."""
+        if self._f_image_needs_close:
+            self._f_image.close()
+        if self._f_bmap_needs_close:
+            self._f_bmap.close()
+
     def _open_image_file(self):
         """Open the image file."""
         try:
@@ -126,56 +213,6 @@ class BmapCreate:
 
         self._f_bmap_needs_close = True
 
-    def __init__(self, image, bmap):
-        """
-        Initialize a class instance:
-        * image - full path or a file-like object of the image to create bmap
-                  for
-        * bmap  - full path or a file object to use for writing the resulting
-                  bmap to
-        """
-
-        self.image_size = None
-        self.image_size_human = None
-        self.block_size = None
-        self.blocks_cnt = None
-        self.mapped_cnt = None
-        self.mapped_size = None
-        self.mapped_size_human = None
-        self.mapped_percent = None
-
-        self._mapped_count_pos1 = None
-        self._mapped_count_pos2 = None
-        self._sha1_pos = None
-
-        self._f_image_needs_close = False
-        self._f_bmap_needs_close = False
-
-        if hasattr(image, "read"):
-            self._f_image = image
-            self._image_path = image.name
-        else:
-            self._image_path = image
-            self._open_image_file()
-
-        if hasattr(bmap, "read"):
-            self._f_bmap = bmap
-            self._bmap_path = bmap.name
-        else:
-            self._bmap_path = bmap
-            self._open_bmap_file()
-
-        self.fiemap = Fiemap.Fiemap(self._f_image)
-
-        self.image_size = self.fiemap.image_size
-        self.image_size_human = human_size(self.image_size)
-        if self.image_size == 0:
-            raise Error("cannot generate bmap for zero-sized image file '%s'"
-                        % self._image_path)
-
-        self.block_size = self.fiemap.block_size
-        self.blocks_cnt = self.fiemap.blocks_cnt
-
     def _bmap_file_start(self):
         """
         A helper function which generates the starting contents of the block
@@ -185,8 +222,6 @@ class BmapCreate:
         # We do not know the amount of mapped blocks at the moment, so just put
         # whitespaces instead of real numbers. Assume the longest possible
         # numbers.
-        mapped_count = ' ' * len(str(self.image_size))
-        mapped_size_human = ' ' * len(self.image_size_human)
 
         xml = _BMAP_START_TEMPLATE \
                % (SUPPORTED_BMAP_VERSION, self.image_size_human,
@@ -196,27 +231,30 @@ class BmapCreate:
         self._f_bmap.write(xml)
         self._mapped_count_pos1 = self._f_bmap.tell()
 
-        # Just put white-spaces instead of real information about mapped blocks
-        xml  = "%s or %.1f    -->\n" % (mapped_size_human, 100.0)
+        xml  = "%s or %s   -->\n" % (' ' * len(self.image_size_human),
+                                   ' ' * len("100.0%"))
         xml += "    <MappedBlocksCount> "
 
         self._f_bmap.write(xml)
         self._mapped_count_pos2 = self._f_bmap.tell()
 
-        xml  = "%s </MappedBlocksCount>\n\n" % mapped_count
+        xml  = "%s </MappedBlocksCount>\n\n" % (' ' * len(str(self.blocks_cnt)))
 
         # pylint: disable=C0301
+        xml += "    <!-- Type of checksum used in this file -->\n"
+        xml += "    <ChecksumType> %s </ChecksumType>\n\n" % self._cs_type
+
         xml += "    <!-- The checksum of this bmap file. When it is calculated, the value of\n"
-        xml += "         the SHA1 checksum has be zero (40 ASCII \"0\" symbols). -->\n"
-        xml += "    <BmapFileSHA1> "
+        xml += "         the checksum has be zero (all ASCII \"0\" symbols).  -->\n"
+        xml += "    <BmapFileChecksum> "
 
         self._f_bmap.write(xml)
-        self._sha1_pos = self._f_bmap.tell()
+        self._chksum_pos = self._f_bmap.tell()
 
-        xml = "0" * 40 + " </BmapFileSHA1>\n\n"
+        xml = "0" * self._cs_len + " </BmapFileChecksum>\n\n"
         xml += "    <!-- The block map which consists of elements which may either be a\n"
-        xml += "         range of blocks or a single block. The 'sha1' attribute (if present)\n"
-        xml += "         is the SHA1 checksum of this blocks range. -->\n"
+        xml += "         range of blocks or a single block. The 'chksum' attribute\n"
+        xml += "         (if present) is the checksum of this blocks range. -->\n"
         xml += "    <BlockMap>\n"
         # pylint: enable=C0301
 
@@ -242,21 +280,23 @@ class BmapCreate:
         self._f_bmap.write("%u" % self.mapped_cnt)
 
         self._f_bmap.seek(0)
-        sha1 = hashlib.sha1(self._f_bmap.read()).hexdigest()
-        self._f_bmap.seek(self._sha1_pos)
-        self._f_bmap.write("%s" % sha1)
+        hash_obj = hashlib.new(self._cs_type)
+        hash_obj.update(self._f_bmap.read())
+        chksum = hash_obj.hexdigest()
+        self._f_bmap.seek(self._chksum_pos)
+        self._f_bmap.write("%s" % chksum)
 
-    def _calculate_sha1(self, first, last):
+    def _calculate_chksum(self, first, last):
         """
-        A helper function which calculates SHA1 checksum for the range of
-        blocks of the image file: from block 'first' to block 'last'.
+        A helper function which calculates checksum for the range of blocks of
+        the image file: from block 'first' to block 'last'.
         """
 
         start = first * self.block_size
         end = (last + 1) * self.block_size
 
         self._f_image.seek(start)
-        hash_obj = hashlib.new("sha1")
+        hash_obj = hashlib.new(self._cs_type)
 
         chunk_size = 1024*1024
         to_read = end - start
@@ -274,7 +314,7 @@ class BmapCreate:
     def generate(self, include_checksums=True):
         """
         Generate bmap for the image file. If 'include_checksums' is 'True',
-        also generate SHA1 checksums for block ranges.
+        also generate checksums for block ranges.
         """
 
         # Save image file position in order to restore it at the end
@@ -285,20 +325,20 @@ class BmapCreate:
         # Generate the block map and write it to the XML block map
         # file as we go.
         self.mapped_cnt = 0
-        for first, last in self.fiemap.get_mapped_ranges(0, self.blocks_cnt):
+        for first, last in self.filemap.get_mapped_ranges(0, self.blocks_cnt):
             self.mapped_cnt += last - first + 1
             if include_checksums:
-                sha1 = self._calculate_sha1(first, last)
-                sha1 = " sha1=\"%s\"" % sha1
+                chksum = self._calculate_chksum(first, last)
+                chksum = " chksum=\"%s\"" % chksum
             else:
-                sha1 = ""
+                chksum = ""
 
             if first != last:
                 self._f_bmap.write("        <Range%s> %s-%s </Range>\n"
-                                   % (sha1, first, last))
+                                   % (chksum, first, last))
             else:
                 self._f_bmap.write("        <Range%s> %s </Range>\n"
-                                   % (sha1, first))
+                                   % (chksum, first))
 
         self.mapped_size = self.mapped_cnt * self.block_size
         self.mapped_size_human = human_size(self.mapped_size)
@@ -313,10 +353,3 @@ class BmapCreate:
                         % (self._bmap_path, err))
 
         self._f_image.seek(image_pos)
-
-    def __del__(self):
-        """The class destructor which closes the opened files."""
-        if self._f_image_needs_close:
-            self._f_image.close()
-        if self._f_bmap_needs_close:
-            self._f_bmap.close()
