@@ -1,3 +1,16 @@
+# Copyright (c) 2012-2013 Intel, Inc.
+# License: GPLv2
+# Author: Artem Bityutskiy <artem.bityutskiy@linux.intel.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2,
+# as published by the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+# General Public License for more details.
+
 """
 This module contains independent functions shared between various
 tests.
@@ -10,15 +23,17 @@ tests.
 import tempfile
 import random
 import itertools
-from bmaptools import BmapHelpers
+import hashlib
+import sys
+from bmaptools import BmapHelpers, BmapCopy, TransRead
 
 def _create_random_sparse_file(file_obj, size):
     """
     Create a sparse file with randomly distributed holes. The mapped areas are
     filled with semi-random data. Returns a tuple containing 2 lists:
-      1. a list of mapped block ranges, same as 'Fiemap.get_mapped_ranges()'
+      1. a list of mapped block ranges, same as 'Filemap.get_mapped_ranges()'
       2. a list of unmapped block ranges (holes), same as
-         'Fiemap.get_unmapped_ranges()'
+         'Filemap.get_unmapped_ranges()'
     """
 
     file_obj.truncate(0)
@@ -98,9 +113,9 @@ def generate_test_files(max_size=4*1024*1024, directory=None, delete=True):
     The generator yields tuples consisting of the following elements:
       1. the test file object
       2. file size in bytes
-      3. a list of mapped block ranges, same as 'Fiemap.get_mapped_ranges()'
+      3. a list of mapped block ranges, same as 'Filemap.get_mapped_ranges()'
       4. a list of unmapped block ranges (holes), same as
-         'Fiemap.get_unmapped_ranges()'
+         'Filemap.get_unmapped_ranges()'
     """
 
     #
@@ -235,3 +250,47 @@ def generate_test_files(max_size=4*1024*1024, directory=None, delete=True):
         blocks_cnt = (size + block_size - 1) / block_size
         yield (file_obj, size, [(0, blocks_cnt - 1)], [])
         file_obj.close()
+
+def calculate_chksum(file_path):
+    """Calculates checksum for the contents of file 'file_path'."""
+
+    file_obj = TransRead.TransRead(file_path)
+    hash_obj = hashlib.new("sha256")
+
+    chunk_size = 1024*1024
+
+    while True:
+        chunk = file_obj.read(chunk_size)
+        if not chunk:
+            break
+        hash_obj.update(chunk)
+
+    file_obj.close()
+    return hash_obj.hexdigest()
+
+def copy_and_verify_image(image, dest, bmap, image_chksum, image_size):
+    """
+    Copy image 'image' using bmap file 'bmap' to the destination file 'dest'
+    and verify the resulting image checksum.
+    """
+
+    f_image = TransRead.TransRead(image)
+    f_dest = open(dest, "w+")
+    if (bmap):
+        f_bmap = open(bmap, "r")
+    else:
+        f_bmap = None
+
+    writer = BmapCopy.BmapCopy(f_image, f_dest, f_bmap, image_size)
+    # Randomly decide whether we want the progress bar or not
+    if bool(random.getrandbits(1)):
+        writer.set_progress_indicator(sys.stdout, None)
+    writer.copy(bool(random.getrandbits(1)), bool(random.getrandbits(1)))
+
+    # Compare the original file and the copy are identical
+    assert calculate_chksum(dest) == image_chksum
+
+    if f_bmap:
+        f_bmap.close()
+    f_dest.close()
+    f_image.close()
